;----------------------------------------------------------------------------
;  Interrupt vector ^O^GVRAM ۑvO V2.30  (C) 2024  ܂
;----------------------------------------------------------------------------
SAVE_GAIJI       equ 1
;SAVE_GVRAM      equ 1
.186
ifdef  SAVE_GAIJI
 TTL equ " Save Interrupt Vectors [ IntVectC ] Version 2.30  Copyright(C) 2020-24 DOSsoft"
else
 TTL equ " Save Interrupt Vectors [ IntVectG ] Version 2.30  Copyright(C) 2020-24 DOSsoft"
endif

ROW_FIXED       equ     15
COLUMN_FIXED    equ      0
SHIFT_KEY       equ     1
ROW             equ     word ptr [bp-04h]
COLUMN          equ     word ptr [bp-06h]

F_HIRESO        = 01h   ; 1high reso[h, 0normal[h
F_RETRO         = 20h   ; 1Ńg@
F_EPSON         = 40h   ; 1EPSON ID or 286@

code            segment public use16
                assume cs:code,ds:code,es:code,ss:code
                org 100h  ; for MS-DOS .COM file
start:  
ifdef   SAVE_GAIJI
        mov     si,0333h
        mov     di,0000h
else
        cmp     si,0333h                ; Check IPLware version (1)
        jb      IncorrectVersion
        sub     di,di                   ; Check IPLware version (2)
        jnz     IncorrectVersion
endif
        call    main_prog
        RETF
IncorrectVersion:
        call    ShortBeep3
        RETF
CPUtype DB      0,0,0
;-------------------------------------
main_prog       PROC   near
                enter   10,0
                call    HiresoCheck
                ; ---- EPSON ------
                call    CheckEPSONPC    ; ID
                call    CheckEPSON286   ; CY=EPSON286  NC=Others
                test    cs:flagset,F_EPSON
                jz      @f
                mov     dx,offset EPSONPC
                call    eputs
                stc
                jmp     ExitMain
        @@:
                mov     ax,ROW_FIXED
                mov     ROW,ax
                mov     ax,COLUMN_FIXED
                mov     COLUMN,ax
                mov     dx,offset TitleString
                call    eputs
                call    WaitShiftKey
                
ifdef   SAVE_GVRAM      
                call    SaveToGVRAM
endif
; Copy IntVect Table to the local area
                push    cs
                pop     es
                mov     cx,400h/2
                mov     di,offset VECT_DATA_AREA  
                xor     ax,ax
                mov     si,ax
                push    ds
                mov     ds,ax
        rep     movsw
                pop     ds


ifdef   SAVE_GAIJI
                test    flagset,F_HIRESO
                jz      @f
                call    SaveToGaiji_H
                jmp     EndMain
        @@:     call    SaveToGaiji
EndMain:        
                mov     dx,offset msg_IV_saved
                call    eputs
                mov     dx,offset msg_code_H
                test    flagset,F_HIRESO
                jnz     @f
                mov     dx,offset msg_code
        @@:     call    eputs
                call    WaitShiftKey
endif
ExitMain:
                leave
                ret
main_prog       ENDP

include epsonpc.inc

ifdef   SAVE_GVRAM
;----------------------------------------------------------------
;       SaveToGVRAM     save to VRAM  E700:0000
;----------------------------------------------------------------
SaveToGVRAM     PROC   near
                push    es
                mov     es,SegGVRAM
; Ivect + sys Work + OS existing below  0000:1000
                mov     cx,400h/2 + 0C00h/2
                xor     ax,ax
                mov     si,ax
                mov     di,ax
                push    ds
                mov     ds,ax
        rep     movsw
                pop     ds
                pop     es
                ret
SaveToGVRAM     ENDP
endif

;---------------------------------------
;       WaitShiftKey
;---------------------------------------
WaitShiftKey    PROC   near
                mov     cx,cs:WaitLoopTime
WaitEnt:        out     5Fh,al
                out     5Fh,al
                out     5Fh,al
                out     5Fh,al
                out     5Fh,al
               loop     WaitEnt
Retry:          mov     ah,02h
                int 18h
                test    al,SHIFT_KEY
                jnz     Retry
                ret
WaitShiftKey    ENDP

;---------------------------------------------------------------------
;       HiresoCheck
;---------------------------------------------------------------------
HiresoCheck     PROC   near
                xor     ax,ax
                mov     cs:Seg0000,ax
                mov     flagset,al
                push    ds
                mov     ds,ax
                mov     al,ds:[0501h]
                pop     ds
                test    al,8
                jz      @f
                or      flagset,F_HIRESO
                mov     cs:SegTVRAM,0E000h
                mov     cs:SegGVRAM,0D700h
        @@:     
                ret
HiresoCheck     ENDP

;---------------------------------------------------------------------------
UploadFontData_SI  PROC   near
                mov     bx,cs
                mov     es,bx
                mov     di,offset FontData32
                mov     cx,32/2
        rep     movsw
                mov     cx,offset FontBuffer
                mov     ah,1Ah
                int     18h
                ret
UploadFontData_SI  ENDP

;---------------------------------------------------------------------------
UploadFontDataN PROC   near
                mov     bx,cs
                mov     es,bx
                mov     di,offset FontData32
                
                push    ds
                mov     si,0
                mov     ds,si
                add     si,ax

                mov     cx,32/2
        rep     movsw

                pop     ds

                mov     cx,offset FontBuffer
                mov     ah,1Ah
                int     18h
                mov     di,offset VECT_DATA_AREA
                ret
UploadFontDataN endp


SaveToGaiji     PROC   near ; 

         ;(32byte=8 D-words)    JIScode
                ; vect 00-07    7770
                mov     dx,7770h
                mov     ax,(00h*4)
                call    UploadFontDataN
                ; vect 08-0F    7771
                mov     dx,7771h
                mov     ax,(08h*4)
                call    UploadFontDataN
                ; vect 10-17    7772
                mov     dx,7772h
                mov     ax,(10h*4)
                call    UploadFontDataN
                ; vect 18-1F    7773
                mov     dx,7773h
                mov     ax,(18h*4)
                call    UploadFontDataN

                ; vect 20-27    7774
                mov     dx,7774h
                mov     ax,(20h*4)
                call    UploadFontDataN

                ; vect 28-2F    7775
                mov     dx,7775h
                mov     ax,(28h*4)
                call    UploadFontDataN

                ; vect 30-37    7776
                mov     dx,7776h
                mov     ax,(30h*4)
                call    UploadFontDataN

                ; vect 38-3F    7777
                mov     dx,7777h
                mov     ax,(38h*4)
                call    UploadFontDataN

;----------------
                mov     bx,cs
                mov     es,bx
                mov     di,offset FontData32

                push    ds      ; gewt work area by 1word
                xor     ax,ax
                mov     ds,ax
                mov     al,ds:[0401h]   ;1
                mov     ah,ds:[0594h]   ;2
                stosw
                mov     al,ds:[05A7h]   ;3
                mov     ah,ds:[05B6h]   ;4 bytes
                stosw
                pop     ds

                mov     dx,7778h      ; byte 8th
                mov     cx,offset FontBuffer
                mov     ah,1Ah
                int     18h
;--------------------------------
                ; SCSI 0460    7779
                mov     dx,7779h
                mov     ax,0460h
                call    UploadFontDataN
                

if 0
                ; vect 40-47    7778
                mov     dx,7778h
                mov     ax,(40h*4)
                call    UploadFontDataN

                ; vect 48-4F    7779
                mov     dx,7779h
                mov     ax,(48h*4)
                call    UploadFontDataN

                ; vect 50-57    777A
                mov     dx,777Ah
                mov     ax,(50h*4)
                call    UploadFontDataN

                ; vect 58-5F    777B
                mov     dx,777Bh
                mov     ax,(58h*4)
                call    UploadFontDataN

                ; vect 60-67    777C
                mov     dx,777Ch
                mov     ax,(60h*4)
                call    UploadFontDataN

                ; vect 68-6F    777D
                mov     dx,777Dh
                mov     ax,(68h*4)
                call    UploadFontDataN

                ; vect 70-77    777E
                mov     dx,777Eh
                mov     ax,(70h*4)
                call    UploadFontDataN

                ; vect 78-7F    777F
                mov     dx,777Fh
                mov     ax,(78h*4)
                call    UploadFontDataN
endif
                ret
SaveToGaiji     ENDP
;---------------------------------------------------------------------------

include hireso.inc

;---------------------------------------------------------------------------
;---------------------------------------
ifdef   SAVE_GVRAM      
TitleString     DB      0E5h,TTL,0
else
TitleString     DB      0A5h,TTL,0
endif

;------------------------------------------------------------------------------
;     eputs  ASCIIZ(ANK)VRAMɓ] input DS:DX pointer, ROW,COLUMN 
;------------------------------------------------------------------------------
eputs           PROC    near
                cld
                mov     es,cs:SegTVRAM
                mov     ax,ROW
                imul    ax,80
                add     ax,COLUMN
                shl     ax,1
                mov     di,ax   ; ES:DI=ɕ\Jnʒuaddressi[
                mov     si,dx
                lodsb           ; Get Color code 1byte
                mov     DL,al   ; Store to DL
StringTop:      lodsb
                cmp     al,0Dh   ; sR[hI
                je      EndString0D
                cmp     al,0Ah   ; AR[hI
                je      EndString0A
                inc     COLUMN
                cmp     COLUMN,80
                jb      @f
                mov     COLUMN,0
                inc     ROW
        @@:     cmp     al,0
                jz      EndString
                mov     ah,0    ; 2022-10-15 ʃoCg00
                mov     es:[di],ax      ; ߂悤ɂ
                mov     es:[di+2000h],DL     ; Color Attribute
                inc     di
                inc     di
                jmp     StringTop
EndString0A:    inc     ROW
EndString0D:    mov     COLUMN,0
EndString:      
                ret
eputs           ENDP


if 0
DebugPrint      PROC   near
                push    ax
                push    bx
                push    cx
                push    dx
                mov     al,cs:PrintTime
                inc     al
                add     cs:PrintTime,al
                mov     bx,offset debmsg0
                inc     byte ptr cs:[bx+2]
                mov     dx,bx
                call    eputs
                mov     al,6
                out     37h,al
                mov     cx,03456h
          @@:   out     5Fh,al
                out     5Fh,al
                out     5Fh,al
                out     5Fh,al
                out     5Fh,al
               loop     @b
                mov     al,7
                out     37h,al
                mov     cx,0FFF7h
          @@:   out     5Fh,al
                out     5Fh,al
                out     5Fh,al
                out     5Fh,al
                out     5Fh,al
                out     5Fh,al
                out     5Fh,al
                out     5Fh,al
                out     5Fh,al
               loop     @b
                pop     dx
                pop     cx
                pop     bx
                pop     ax
                ret
PrintTime       DB      0
debmsg0         DB      0C5h," 0 ",0
DebugPrint      ENDP
endif

;---------------------------------
;       ShortBeep
;---------------------------------
ShortWait       PROC   near
                push    cx
                push    ax
                mov     cx,6543
                jmp     ShortWaitEnt
ShortWait       ENDP

ShortBeep       PROC   near
                push    cx
                push    ax
                mov     al,6
                out     37h,al
                mov     cx,1234
ShortWaitEnt:
        @@:     out     5Fh,al
                out     5Fh,al
                out     5Fh,al
                out     5Fh,al
                out     5Fh,al
               loop     @b
                mov     al,7
                out     37h,al
                pop     ax
                pop     cx
                ret
ShortBeep       ENDP

ShortBeep3      PROC   near
                call    ShortWait
                call    ShortBeep
                call    ShortWait
                call    ShortBeep
                call    ShortWait
                call    ShortBeep
                ret
ShortBeep3      ENDP


WaitLoopTime    DW      7777h
flagset         DB      0
msg_IV_saved    DB      0E1h," The Interrupt vector table was saved to GAIJI code ",0
msg_code_H      DB      0E1h,"7650h--7657h.",13,10,0
msg_code        DB      0E1h,"7770h--7778h.",13,10,0
EPSONPC         DB      041h," EPSON PC may not reboot.",0

align 16
Seg0000         DW      0000h
SegTVRAM        DW      0A000h
SegGVRAM        DW      0E700h
FontBuffer      DW      00h
FontData72      label   word
FontData32      DW      16 dup(0000h)
                DW      20 dup(0000h)   ;   =  db  72 dup (0)
                DB      "Interrupt vector"
VECT_DATA_AREA  DD      100h   dup (?)
;---------------------------------------

END_OF_DATA     label   byte

code    ends
        end     start

